@@ -16,6 +16,8 @@
  */
 package org.apache.lucene.codecs.lucene90.blocktree;
 
+import static org.apache.lucene.util.fst.FSTCompiler.getOnHeapReaderWriter;
+
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -52,6 +54,7 @@ import org.apache.lucene.util.fst.BytesRefFSTEnum;
 import org.apache.lucene.util.fst.FST;
 import org.apache.lucene.util.fst.FSTCompiler;
 import org.apache.lucene.util.fst.Util;
+import org.apache.lucene.util.packed.PackedInts;
 
 /*
   TODO:
@@ -235,6 +238,7 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
   final int maxDoc;
   final int minItemsInBlock;
   final int maxItemsInBlock;
+  final int version;
 
   final PostingsWriterBase postingsWriter;
   final FieldInfos fieldInfos;
@@ -252,10 +256,37 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
       int minItemsInBlock,
       int maxItemsInBlock)
       throws IOException {
+    this(
+        state,
+        postingsWriter,
+        minItemsInBlock,
+        maxItemsInBlock,
+        Lucene90BlockTreeTermsReader.VERSION_CURRENT);
+  }
+
+  /** Expert constructor that allows configuring the version, used for bw tests. */
+  public Lucene90BlockTreeTermsWriter(
+      SegmentWriteState state,
+      PostingsWriterBase postingsWriter,
+      int minItemsInBlock,
+      int maxItemsInBlock,
+      int version)
+      throws IOException {
     validateSettings(minItemsInBlock, maxItemsInBlock);
 
     this.minItemsInBlock = minItemsInBlock;
     this.maxItemsInBlock = maxItemsInBlock;
+    if (version < Lucene90BlockTreeTermsReader.VERSION_START
+        || version > Lucene90BlockTreeTermsReader.VERSION_CURRENT) {
+      throw new IllegalArgumentException(
+          "Expected version in range ["
+              + Lucene90BlockTreeTermsReader.VERSION_START
+              + ", "
+              + Lucene90BlockTreeTermsReader.VERSION_CURRENT
+              + "], but got "
+              + version);
+    }
+    this.version = version;
 
     this.maxDoc = state.segmentInfo.maxDoc();
     this.fieldInfos = state.fieldInfos;
@@ -273,7 +304,7 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
       CodecUtil.writeIndexHeader(
           termsOut,
           Lucene90BlockTreeTermsReader.TERMS_CODEC_NAME,
-          Lucene90BlockTreeTermsReader.VERSION_CURRENT,
+          version,
           state.segmentInfo.getId(),
           state.segmentSuffix);
 
@@ -286,7 +317,7 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
       CodecUtil.writeIndexHeader(
           indexOut,
           Lucene90BlockTreeTermsReader.TERMS_INDEX_CODEC_NAME,
-          Lucene90BlockTreeTermsReader.VERSION_CURRENT,
+          version,
           state.segmentInfo.getId(),
           state.segmentSuffix);
       // segment = state.segmentInfo.name;
@@ -300,7 +331,7 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
       CodecUtil.writeIndexHeader(
           metaOut,
           Lucene90BlockTreeTermsReader.TERMS_META_CODEC_NAME,
-          Lucene90BlockTreeTermsReader.VERSION_CURRENT,
+          version,
           state.segmentInfo.getId(),
           state.segmentSuffix);
 
@@ -429,7 +460,26 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
     return brToString(new BytesRef(b));
   }
 
-  private static final class PendingBlock extends PendingEntry {
+  /**
+   * Encodes long value to variable length byte[], in MSB order. Use {@link
+   * FieldReader#readMSBVLong} to decode.
+   *
+   * <p>Package private for testing
+   */
+  static void writeMSBVLong(long l, DataOutput scratchBytes) throws IOException {
+    assert l >= 0;
+    // Keep zero bits on most significant byte to have more chance to get prefix bytes shared.
+    // e.g. we expect 0x7FFF stored as [0x81, 0xFF, 0x7F] but not [0xFF, 0xFF, 0x40]
+    final int bytesNeeded = (Long.SIZE - Long.numberOfLeadingZeros(l) - 1) / 7 + 1;
+    l <<= Long.SIZE - bytesNeeded * 7;
+    for (int i = 1; i < bytesNeeded; i++) {
+      scratchBytes.writeByte((byte) (((l >>> 57) & 0x7FL) | 0x80));
+      l = l << 7;
+    }
+    scratchBytes.writeByte((byte) (((l >>> 57) & 0x7FL)));
+  }
+
+  private final class PendingBlock extends PendingEntry {
     public final BytesRef prefix;
     public final long fp;
     public FST<BytesRef> index;
@@ -471,10 +521,12 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
 
       assert scratchBytes.size() == 0;
 
-      // TODO: try writing the leading vLong in MSB order
-      // (opposite of what Lucene does today), for better
-      // outputs sharing in the FST
-      scratchBytes.writeVLong(encodeOutput(fp, hasTerms, isFloor));
+      // write the leading vLong in MSB order for better outputs sharing in the FST
+      if (version >= Lucene90BlockTreeTermsReader.VERSION_MSB_VLONG_OUTPUT) {
+        writeMSBVLong(encodeOutput(fp, hasTerms, isFloor), scratchBytes);
+      } else {
+        scratchBytes.writeVLong(encodeOutput(fp, hasTerms, isFloor));
+      }
       if (isFloor) {
         scratchBytes.writeVInt(blocks.size() - 1);
         for (int i = 1; i < blocks.size(); i++) {
@@ -490,10 +542,31 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
         }
       }
 
+      long estimateSize = prefix.length;
+      for (PendingBlock block : blocks) {
+        if (block.subIndices != null) {
+          for (FST<BytesRef> subIndex : block.subIndices) {
+            estimateSize += subIndex.numBytes();
+          }
+        }
+      }
+      int estimateBitsRequired = PackedInts.bitsRequired(estimateSize);
+      int pageBits = Math.min(15, Math.max(6, estimateBitsRequired));
+
       final ByteSequenceOutputs outputs = ByteSequenceOutputs.getSingleton();
+      final int fstVersion;
+      if (version >= Lucene90BlockTreeTermsReader.VERSION_CURRENT) {
+        fstVersion = FST.VERSION_CURRENT;
+      } else {
+        fstVersion = FST.VERSION_90;
+      }
       final FSTCompiler<BytesRef> fstCompiler =
           new FSTCompiler.Builder<>(FST.INPUT_TYPE.BYTE1, outputs)
-              .shouldShareNonSingletonNodes(false)
+              // Disable suffixes sharing for block tree index because suffixes are mostly dropped
+              // from the FST index and left in the term blocks.
+              .suffixRAMLimitMB(0d)
+              .dataOutput(getOnHeapReaderWriter(pageBits))
+              .setVersion(fstVersion)
               .build();
       // if (DEBUG) {
       //  System.out.println("  compile index for prefix=" + prefix);
@@ -514,7 +587,7 @@ public final class Lucene90BlockTreeTermsWriter extends FieldsConsumer {
         }
       }
 
-      index = fstCompiler.compile();
+      index = FST.fromFSTReader(fstCompiler.compile(), fstCompiler.getFSTReader());
 
       assert subIndices == null;
 
