@@ -17,7 +17,6 @@
 package org.apache.lucene.document;
 
 import java.io.IOException;
-import java.util.Arrays;
 import java.util.Objects;
 import org.apache.lucene.geo.GeoEncodingUtils;
 import org.apache.lucene.geo.GeoUtils;
@@ -387,17 +386,12 @@ final class LatLonPointDistanceFeatureQuery extends Query {
       // Ideally we would be doing a distance query but that is too expensive so we approximate
       // with a box query which performs better.
       Rectangle box = Rectangle.fromPointDistance(originLat, originLon, maxDistance);
-      final byte[] minLat = new byte[LatLonPoint.BYTES];
-      final byte[] maxLat = new byte[LatLonPoint.BYTES];
-      final byte[] minLon = new byte[LatLonPoint.BYTES];
-      final byte[] maxLon = new byte[LatLonPoint.BYTES];
+      final int minLat = GeoEncodingUtils.encodeLatitude(box.minLat);
+      final int maxLat = GeoEncodingUtils.encodeLatitude(box.maxLat);
+      final int minLon = GeoEncodingUtils.encodeLongitude(box.minLon);
+      final int maxLon = GeoEncodingUtils.encodeLongitude(box.maxLon);
       final boolean crossDateLine = box.crossesDateline();
 
-      NumericUtils.intToSortableBytes(GeoEncodingUtils.encodeLatitude(box.minLat), minLat, 0);
-      NumericUtils.intToSortableBytes(GeoEncodingUtils.encodeLatitude(box.maxLat), maxLat, 0);
-      NumericUtils.intToSortableBytes(GeoEncodingUtils.encodeLongitude(box.minLon), minLon, 0);
-      NumericUtils.intToSortableBytes(GeoEncodingUtils.encodeLongitude(box.maxLon), maxLon, 0);
-
       DocIdSetBuilder result = new DocIdSetBuilder(maxDoc);
       final int doc = docID();
       IntersectVisitor visitor =
@@ -425,53 +419,20 @@ final class LatLonPointDistanceFeatureQuery extends Query {
                 // Already visited or skipped
                 return;
               }
-              if (Arrays.compareUnsigned(
-                          packedValue, 0, LatLonPoint.BYTES, maxLat, 0, LatLonPoint.BYTES)
-                      > 0
-                  || Arrays.compareUnsigned(
-                          packedValue, 0, LatLonPoint.BYTES, minLat, 0, LatLonPoint.BYTES)
-                      < 0) {
+              int lat = NumericUtils.sortableBytesToInt(packedValue, 0);
+              if (lat > maxLat || lat < minLat) {
                 // Latitude out of range
                 return;
               }
+              int lon = NumericUtils.sortableBytesToInt(packedValue, LatLonPoint.BYTES);
               if (crossDateLine) {
-                if (Arrays.compareUnsigned(
-                            packedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            minLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        < 0
-                    && Arrays.compareUnsigned(
-                            packedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            maxLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        > 0) {
+                if (lon < minLon && lon > maxLon) {
                   // Longitude out of range
                   return;
                 }
 
               } else {
-                if (Arrays.compareUnsigned(
-                            packedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            maxLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        > 0
-                    || Arrays.compareUnsigned(
-                            packedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            minLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        < 0) {
+                if (lon > maxLon || lon < minLon) {
                   // Longitude out of range
                   return;
                 }
@@ -481,96 +442,27 @@ final class LatLonPointDistanceFeatureQuery extends Query {
 
             @Override
             public Relation compare(byte[] minPackedValue, byte[] maxPackedValue) {
-
-              if (Arrays.compareUnsigned(
-                          minPackedValue, 0, LatLonPoint.BYTES, maxLat, 0, LatLonPoint.BYTES)
-                      > 0
-                  || Arrays.compareUnsigned(
-                          maxPackedValue, 0, LatLonPoint.BYTES, minLat, 0, LatLonPoint.BYTES)
-                      < 0) {
+              int latLowerBound = NumericUtils.sortableBytesToInt(minPackedValue, 0);
+              int latUpperBound = NumericUtils.sortableBytesToInt(maxPackedValue, 0);
+              if (latLowerBound > maxLat || latUpperBound < minLat) {
                 return Relation.CELL_OUTSIDE_QUERY;
               }
-              boolean crosses =
-                  Arrays.compareUnsigned(
-                              minPackedValue, 0, LatLonPoint.BYTES, minLat, 0, LatLonPoint.BYTES)
-                          < 0
-                      || Arrays.compareUnsigned(
-                              maxPackedValue, 0, LatLonPoint.BYTES, maxLat, 0, LatLonPoint.BYTES)
-                          > 0;
-
+              boolean crosses = latLowerBound < minLat || latUpperBound > maxLat;
+              int lonLowerBound =
+                  NumericUtils.sortableBytesToInt(minPackedValue, LatLonPoint.BYTES);
+              int lonUpperBound =
+                  NumericUtils.sortableBytesToInt(maxPackedValue, LatLonPoint.BYTES);
               if (crossDateLine) {
-                if (Arrays.compareUnsigned(
-                            minPackedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            maxLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        > 0
-                    && Arrays.compareUnsigned(
-                            maxPackedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            minLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        < 0) {
+                if (lonLowerBound > maxLon && lonUpperBound < minLon) {
                   return Relation.CELL_OUTSIDE_QUERY;
                 }
-                crosses |=
-                    Arrays.compareUnsigned(
-                                minPackedValue,
-                                LatLonPoint.BYTES,
-                                2 * LatLonPoint.BYTES,
-                                maxLon,
-                                0,
-                                LatLonPoint.BYTES)
-                            < 0
-                        || Arrays.compareUnsigned(
-                                maxPackedValue,
-                                LatLonPoint.BYTES,
-                                2 * LatLonPoint.BYTES,
-                                minLon,
-                                0,
-                                LatLonPoint.BYTES)
-                            > 0;
+                crosses |= lonLowerBound < maxLon || lonUpperBound > minLon;
 
               } else {
-                if (Arrays.compareUnsigned(
-                            minPackedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            maxLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        > 0
-                    || Arrays.compareUnsigned(
-                            maxPackedValue,
-                            LatLonPoint.BYTES,
-                            2 * LatLonPoint.BYTES,
-                            minLon,
-                            0,
-                            LatLonPoint.BYTES)
-                        < 0) {
+                if (lonLowerBound > maxLon || lonUpperBound < minLon) {
                   return Relation.CELL_OUTSIDE_QUERY;
                 }
-                crosses |=
-                    Arrays.compareUnsigned(
-                                minPackedValue,
-                                LatLonPoint.BYTES,
-                                2 * LatLonPoint.BYTES,
-                                minLon,
-                                0,
-                                LatLonPoint.BYTES)
-                            < 0
-                        || Arrays.compareUnsigned(
-                                maxPackedValue,
-                                LatLonPoint.BYTES,
-                                2 * LatLonPoint.BYTES,
-                                maxLon,
-                                0,
-                                LatLonPoint.BYTES)
-                            > 0;
+                crosses |= lonLowerBound < minLon || lonUpperBound > maxLon;
               }
               if (crosses) {
                 return Relation.CELL_CROSSES_QUERY;
