@@ -161,6 +161,7 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             "test",
             10,
             false,
+            false,
             Codec.getDefault(),
             Collections.emptyMap(),
             StringHelper.randomId(),
@@ -198,7 +199,8 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             0,
             VectorEncoding.FLOAT32,
             VectorSimilarityFunction.EUCLIDEAN,
-            true);
+            true,
+            false);
     List<Integer> docsDeleted = Arrays.asList(1, 3, 7, 8, DocIdSetIterator.NO_MORE_DOCS);
     List<DocValuesFieldUpdates> updates = Arrays.asList(singleUpdate(docsDeleted, 10, true));
     for (DocValuesFieldUpdates update : updates) {
@@ -236,7 +238,8 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             0,
             VectorEncoding.FLOAT32,
             VectorSimilarityFunction.EUCLIDEAN,
-            true);
+            true,
+            false);
     for (DocValuesFieldUpdates update : updates) {
       deletes.onDocValuesUpdate(fieldInfo, update.iterator());
     }
@@ -262,6 +265,7 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             newIndexWriterConfig()
                 .setSoftDeletesField("_soft_deletes")
                 .setMaxBufferedDocs(3) // make sure we write one segment
+                .setMergePolicy(NoMergePolicy.INSTANCE) // prevent deletes from triggering merges
                 .setRAMBufferSizeMB(IndexWriterConfig.DISABLE_AUTO_FLUSH));
     Document doc = new Document();
     doc.add(new StringField("id", "1", Field.Store.YES));
@@ -299,7 +303,8 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             0,
             VectorEncoding.FLOAT32,
             VectorSimilarityFunction.EUCLIDEAN,
-            true);
+            true,
+            false);
     List<Integer> docsDeleted = Arrays.asList(1, DocIdSetIterator.NO_MORE_DOCS);
     List<DocValuesFieldUpdates> updates = Arrays.asList(singleUpdate(docsDeleted, 3, true));
     for (DocValuesFieldUpdates update : updates) {
@@ -330,6 +335,7 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             newIndexWriterConfig()
                 .setSoftDeletesField("_soft_deletes")
                 .setMaxBufferedDocs(3) // make sure we write one segment
+                .setMergePolicy(NoMergePolicy.INSTANCE) // prevent deletes from triggering merges
                 .setRAMBufferSizeMB(IndexWriterConfig.DISABLE_AUTO_FLUSH));
     Document doc = new Document();
     doc.add(new StringField("id", "1", Field.Store.YES));
@@ -367,7 +373,8 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             0,
             VectorEncoding.FLOAT32,
             VectorSimilarityFunction.EUCLIDEAN,
-            true);
+            true,
+            false);
     List<DocValuesFieldUpdates> updates =
         Arrays.asList(singleUpdate(Arrays.asList(0, 1, DocIdSetIterator.NO_MORE_DOCS), 3, false));
     for (DocValuesFieldUpdates update : updates) {
@@ -404,7 +411,8 @@ public class TestPendingSoftDeletes extends TestPendingDeletes {
             0,
             VectorEncoding.FLOAT32,
             VectorSimilarityFunction.EUCLIDEAN,
-            true);
+            true,
+            false);
     updates = Arrays.asList(singleUpdate(Arrays.asList(1, DocIdSetIterator.NO_MORE_DOCS), 3, true));
     for (DocValuesFieldUpdates update : updates) {
       deletes.onDocValuesUpdate(fieldInfo, update.iterator());
