@@ -36,6 +36,7 @@ import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.LeafReaderContext;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.search.similarities.ClassicSimilarity;
 import org.apache.lucene.search.similarities.Similarity;
@@ -178,7 +179,8 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
     final Scorer ds = dw.scorer(context);
     final boolean skipOk = ds.iterator().advance(3) != DocIdSetIterator.NO_MORE_DOCS;
     if (skipOk) {
-      fail("firsttime skipTo found a match? ... " + r.document(ds.docID()).get("id"));
+      fail(
+          "firsttime skipTo found a match? ... " + r.storedFields().document(ds.docID()).get("id"));
     }
   }
 
@@ -195,7 +197,7 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
     assertTrue(
         "firsttime skipTo found no match",
         ds.iterator().advance(3) != DocIdSetIterator.NO_MORE_DOCS);
-    assertEquals("found wrong docid", "d4", r.document(ds.docID()).get("id"));
+    assertEquals("found wrong docid", "d4", r.storedFields().document(ds.docID()).get("id"));
   }
 
   public void testSimpleEqualScores1() throws Exception {
@@ -275,7 +277,7 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
 
     try {
       assertEquals("3 docs should match " + q.toString(), 3, h.length);
-      assertEquals("wrong first", "d2", s.doc(h[0].doc).get("id"));
+      assertEquals("wrong first", "d2", s.storedFields().document(h[0].doc).get("id"));
       float score0 = h[0].score;
       float score1 = h[1].score;
       float score2 = h[2].score;
@@ -347,7 +349,7 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
         /* note: -1 */
         assertEquals("score #" + i + " is not the same", score, h[i].score, SCORE_COMP_THRESH);
       }
-      assertEquals("wrong last", "d1", s.doc(h[h.length - 1].doc).get("id"));
+      assertEquals("wrong last", "d1", s.storedFields().document(h[h.length - 1].doc).get("id"));
       float score1 = h[h.length - 1].score;
       assertTrue(
           "d1 does not have worse score then others: " + score + " >? " + score1, score > score1);
@@ -384,10 +386,10 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
       float score2 = h[2].score;
       float score3 = h[3].score;
 
-      String doc0 = s.doc(h[0].doc).get("id");
-      String doc1 = s.doc(h[1].doc).get("id");
-      String doc2 = s.doc(h[2].doc).get("id");
-      String doc3 = s.doc(h[3].doc).get("id");
+      String doc0 = s.storedFields().document(h[0].doc).get("id");
+      String doc1 = s.storedFields().document(h[1].doc).get("id");
+      String doc2 = s.storedFields().document(h[2].doc).get("id");
+      String doc3 = s.storedFields().document(h[3].doc).get("id");
 
       assertTrue("doc0 should be d2 or d4: " + doc0, doc0.equals("d2") || doc0.equals("d4"));
       assertTrue("doc1 should be d2 or d4: " + doc0, doc1.equals("d2") || doc1.equals("d4"));
@@ -435,10 +437,10 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
       float score2 = h[2].score;
       float score3 = h[3].score;
 
-      String doc0 = s.doc(h[0].doc).get("id");
-      String doc1 = s.doc(h[1].doc).get("id");
-      String doc2 = s.doc(h[2].doc).get("id");
-      String doc3 = s.doc(h[3].doc).get("id");
+      String doc0 = s.storedFields().document(h[0].doc).get("id");
+      String doc1 = s.storedFields().document(h[1].doc).get("id");
+      String doc2 = s.storedFields().document(h[2].doc).get("id");
+      String doc3 = s.storedFields().document(h[3].doc).get("id");
 
       assertEquals("doc0 should be d4: ", "d4", doc0);
       assertEquals("doc1 should be d3: ", "d3", doc1);
@@ -580,8 +582,9 @@ public class TestDisjunctionMaxQuery extends LuceneTestCase {
     DecimalFormat f =
         new DecimalFormat("0.000000000", DecimalFormatSymbols.getInstance(Locale.ROOT));
 
+    StoredFields storedFields = searcher.storedFields();
     for (int i = 0; i < h.length; i++) {
-      Document d = searcher.doc(h[i].doc);
+      Document d = storedFields.document(h[i].doc);
       float score = h[i].score;
       System.err.println("#" + i + ": " + f.format(score) + " - " + d.get("id"));
     }
