@@ -42,8 +42,9 @@ import org.apache.lucene.search.ScoreDoc;
 import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.TopDocs;
 import org.apache.lucene.search.TopFieldCollector;
+import org.apache.lucene.search.TopFieldCollectorManager;
 import org.apache.lucene.search.TopFieldDocs;
-import org.apache.lucene.search.TopScoreDocCollector;
+import org.apache.lucene.search.TopScoreDocCollectorManager;
 import org.apache.lucene.util.ThreadInterruptedException;
 
 /**
@@ -133,7 +134,12 @@ public class DrillSideways {
   /**
    * Subclass can override to customize drill down facets collector. Returning {@code null} is valid
    * if no drill down facet collection is needed.
+   *
+   * @deprecated This is only used by the deprecated {@link #search(DrillDownQuery, Collector)}
+   *     entry-point. Please use {@link #search(DrillDownQuery, CollectorManager)} instead, and
+   *     leverage {@link #createDrillDownFacetsCollectorManager()} as necessary
    */
+  @Deprecated
   protected FacetsCollector createDrillDownFacetsCollector() {
     return new FacetsCollector();
   }
@@ -146,6 +152,14 @@ public class DrillSideways {
     return new FacetsCollectorManager();
   }
 
+  /**
+   * Subclass can override to customize drill sideways facets collector. This should not return
+   * {@code null} as we assume drill sideways is being used to collect "sideways" hits:
+   */
+  protected FacetsCollectorManager createDrillSidewaysFacetsCollectorManager() {
+    return new FacetsCollectorManager();
+  }
+
   /** Subclass can override to customize per-dim Facets impl. */
   protected Facets buildFacetsResult(
       FacetsCollector drillDowns, FacetsCollector[] drillSideways, String[] drillSidewaysDims)
@@ -287,12 +301,12 @@ public class DrillSideways {
         limit = 1; // the collector does not alow numHits = 0
       }
       final int fTopN = Math.min(topN, limit);
-
-      final CollectorManager<TopFieldCollector, TopFieldDocs> collectorManager =
-          TopFieldCollector.createSharedManager(sort, fTopN, after, Integer.MAX_VALUE);
+      final boolean supportsConcurrency = searcher.getSlices().length > 1;
+      final TopFieldCollectorManager collectorManager =
+          new TopFieldCollectorManager(sort, fTopN, after, Integer.MAX_VALUE, supportsConcurrency);
       final ConcurrentDrillSidewaysResult<TopFieldDocs> r = search(query, collectorManager);
-
       TopFieldDocs topDocs = r.collectorResult;
+
       if (doDocScores) {
         TopFieldCollector.populateScores(topDocs.scoreDocs, searcher, query);
       }
@@ -320,9 +334,9 @@ public class DrillSideways {
       limit = 1; // the collector does not alow numHits = 0
     }
     final int fTopN = Math.min(topN, limit);
-
-    final CollectorManager<TopScoreDocCollector, TopDocs> collectorManager =
-        TopScoreDocCollector.createSharedManager(fTopN, after, Integer.MAX_VALUE);
+    final boolean supportsConcurrency = searcher.getSlices().length > 1;
+    final TopScoreDocCollectorManager collectorManager =
+        new TopScoreDocCollectorManager(fTopN, after, Integer.MAX_VALUE, supportsConcurrency);
     final ConcurrentDrillSidewaysResult<TopDocs> r = search(query, collectorManager);
     return new DrillSidewaysResult(
         r.facets,
@@ -496,7 +510,7 @@ public class DrillSideways {
     FacetsCollectorManager[] drillSidewaysFacetsCollectorManagers =
         new FacetsCollectorManager[numDims];
     for (int i = 0; i < numDims; i++) {
-      drillSidewaysFacetsCollectorManagers[i] = new FacetsCollectorManager();
+      drillSidewaysFacetsCollectorManagers[i] = createDrillSidewaysFacetsCollectorManager();
     }
 
     DrillSidewaysQuery dsq =
@@ -566,7 +580,10 @@ public class DrillSideways {
     for (String dim : drillDownDims.keySet())
       callableCollectors.add(
           new CallableCollector(
-              i++, searcher, getDrillDownQuery(query, filters, dim), new FacetsCollectorManager()));
+              i++,
+              searcher,
+              getDrillDownQuery(query, filters, dim),
+              createDrillSidewaysFacetsCollectorManager()));
 
     final FacetsCollector mainFacetsCollector;
     final FacetsCollector[] facetsCollectors = new FacetsCollector[drillDownDims.size()];
