@@ -17,6 +17,7 @@
 
 package org.apache.lucene.facet;
 
+import com.carrotsearch.randomizedtesting.generators.RandomNumbers;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
@@ -394,12 +395,12 @@ public class TestLongValueFacetCounts extends FacetTestCase {
       if (random().nextBoolean()) {
         topN = docCount;
       } else {
-        topN = random().nextInt(docCount);
+        topN = RandomNumbers.randomIntBetween(random(), 1, docCount);
       }
       if (VERBOSE) {
         System.out.println("  topN=" + topN);
       }
-      actual = facetCounts.getTopChildrenSortByCount(topN);
+      actual = facetCounts.getTopChildren(topN, "field");
       assertSame(
           "all docs, sort facets by count",
           expectedCounts,
@@ -486,9 +487,9 @@ public class TestLongValueFacetCounts extends FacetTestCase {
       if (random().nextBoolean()) {
         topN = docCount;
       } else {
-        topN = random().nextInt(docCount);
+        topN = RandomNumbers.randomIntBetween(random(), 1, docCount);
       }
-      actual = facetCounts.getTopChildrenSortByCount(topN);
+      actual = facetCounts.getTopChildren(topN, "field");
       assertSame(
           "id " + minId + "-" + maxId + ", sort facets by count",
           expectedCounts,
@@ -657,12 +658,12 @@ public class TestLongValueFacetCounts extends FacetTestCase {
       if (random().nextBoolean()) {
         topN = docCount;
       } else {
-        topN = random().nextInt(docCount);
+        topN = RandomNumbers.randomIntBetween(random(), 1, docCount);
       }
       if (VERBOSE) {
         System.out.println("  topN=" + topN);
       }
-      actual = facetCounts.getTopChildrenSortByCount(topN);
+      actual = facetCounts.getTopChildren(topN, "field");
       assertSame(
           "all docs, sort facets by count",
           expectedCounts,
@@ -727,9 +728,9 @@ public class TestLongValueFacetCounts extends FacetTestCase {
       if (random().nextBoolean()) {
         topN = docCount;
       } else {
-        topN = random().nextInt(docCount);
+        topN = RandomNumbers.randomIntBetween(random(), 1, docCount);
       }
-      actual = facetCounts.getTopChildrenSortByCount(topN);
+      actual = facetCounts.getTopChildren(topN, "field");
       assertSame(
           "id " + minId + "-" + maxId + ", sort facets by count",
           expectedCounts,
