@@ -57,7 +57,7 @@ import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.SortedNumericSortField;
 import org.apache.lucene.search.SortedSetSortField;
 import org.apache.lucene.search.TopDocs;
-import org.apache.lucene.search.TopScoreDocCollector;
+import org.apache.lucene.search.TopScoreDocCollectorManager;
 import org.apache.lucene.search.TotalHits;
 import org.apache.lucene.search.similarities.BM25Similarity;
 import org.apache.lucene.search.similarities.ClassicSimilarity;
@@ -152,7 +152,7 @@ public final class SearchImpl extends LukeModel implements Search {
 
     if (rewrite) {
       try {
-        query = query.rewrite(reader);
+        query = query.rewrite(searcher);
       } catch (IOException e) {
         throw new LukeException(
             String.format(Locale.ENGLISH, "Failed to rewrite query: %s", query.toString()), e);
@@ -250,15 +250,15 @@ public final class SearchImpl extends LukeModel implements Search {
 
   @Override
   public Query mltQuery(int docid, MLTConfig mltConfig, Analyzer analyzer) {
-    MoreLikeThis mlt = new MoreLikeThis(reader);
-
-    mlt.setAnalyzer(analyzer);
-    mlt.setFieldNames(mltConfig.getFieldNames());
-    mlt.setMinDocFreq(mltConfig.getMinDocFreq());
-    mlt.setMaxDocFreq(mltConfig.getMaxDocFreq());
-    mlt.setMinTermFreq(mltConfig.getMinTermFreq());
-
     try {
+      MoreLikeThis mlt = new MoreLikeThis(reader);
+
+      mlt.setAnalyzer(analyzer);
+      mlt.setFieldNames(mltConfig.getFieldNames());
+      mlt.setMinDocFreq(mltConfig.getMinDocFreq());
+      mlt.setMaxDocFreq(mltConfig.getMaxDocFreq());
+      mlt.setMinTermFreq(mltConfig.getMinTermFreq());
+
       return mlt.like(docid);
     } catch (IOException e) {
       throw new LukeException("Failed to create MLT query for doc: " + docid, e);
@@ -314,9 +314,10 @@ public final class SearchImpl extends LukeModel implements Search {
       topDocs = searcher.searchAfter(after, query, pageSize, sort);
     } else {
       int hitsThreshold = exactHitsCount ? Integer.MAX_VALUE : DEFAULT_TOTAL_HITS_THRESHOLD;
-      TopScoreDocCollector collector = TopScoreDocCollector.create(pageSize, after, hitsThreshold);
-      searcher.search(query, collector);
-      topDocs = collector.topDocs();
+      TopScoreDocCollectorManager collectorManager =
+          new TopScoreDocCollectorManager(
+              pageSize, after, hitsThreshold, searcher.getSlices().length > 1);
+      topDocs = searcher.search(query, collectorManager);
     }
 
     // reset total hits for the current query
