@@ -162,7 +162,7 @@ public class TestMemoryIndexAgainstDirectory extends BaseTokenStreamTestCase {
   }
 
   private void duellReaders(CompositeReader other, LeafReader memIndexReader) throws IOException {
-    Fields memFields = memIndexReader.getTermVectors(0);
+    Fields memFields = memIndexReader.termVectors().get(0);
     for (String field : FieldInfos.getIndexedFields(other)) {
       Terms memTerms = memFields.terms(field);
       Terms iwTerms = memIndexReader.terms(field);
@@ -283,7 +283,6 @@ public class TestMemoryIndexAgainstDirectory extends BaseTokenStreamTestCase {
       }
     }
   }
-  ;
 
   /**
    * Some terms to be indexed, in addition to random words. These terms are commonly used in the
@@ -706,10 +705,10 @@ public class TestMemoryIndexAgainstDirectory extends BaseTokenStreamTestCase {
     memIndex.addField(field_name, "foo bar foo bar foo", mockAnalyzer);
 
     // compare term vectors
-    Terms ramTv = reader.getTermVector(0, field_name);
+    Terms ramTv = reader.termVectors().get(0, field_name);
     IndexReader memIndexReader = memIndex.createSearcher().getIndexReader();
     TestUtil.checkReader(memIndexReader);
-    Terms memTv = memIndexReader.getTermVector(0, field_name);
+    Terms memTv = memIndexReader.termVectors().get(0, field_name);
 
     compareTermVectors(ramTv, memTv, field_name);
     memIndexReader.close();
