@@ -64,7 +64,7 @@ import org.apache.lucene.search.SearcherManager;
 import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.TermQuery;
-import org.apache.lucene.search.TopFieldCollector;
+import org.apache.lucene.search.TopFieldCollectorManager;
 import org.apache.lucene.search.TopFieldDocs;
 import org.apache.lucene.search.suggest.InputIterator;
 import org.apache.lucene.search.suggest.Lookup;
@@ -116,6 +116,7 @@ public class AnalyzingInfixSuggester extends Lookup implements Closeable {
 
   /** Analyzer used at search time */
   protected final Analyzer queryAnalyzer;
+
   /** Analyzer used at index time */
   protected final Analyzer indexAnalyzer;
 
@@ -727,7 +728,6 @@ public class AnalyzingInfixSuggester extends Lookup implements Closeable {
     // System.out.println("finalQuery=" + finalQuery);
 
     // Sort by weight, descending:
-    TopFieldCollector c = TopFieldCollector.create(SORT, num, 1);
     List<LookupResult> results = null;
     SearcherManager mgr;
     IndexSearcher searcher;
@@ -739,10 +739,10 @@ public class AnalyzingInfixSuggester extends Lookup implements Closeable {
       searcherMgrReadLock.unlock();
     }
     try {
+      TopFieldCollectorManager c =
+          new TopFieldCollectorManager(SORT, num, null, 1, searcher.getSlices().length > 1);
       // System.out.println("got searcher=" + searcher);
-      searcher.search(finalQuery, c);
-
-      TopFieldDocs hits = c.topDocs();
+      TopFieldDocs hits = searcher.search(finalQuery, c);
 
       // Slower way if postings are not pre-sorted by weight:
       // hits = searcher.search(query, null, num, SORT);
@@ -751,7 +751,7 @@ public class AnalyzingInfixSuggester extends Lookup implements Closeable {
       mgr.release(searcher);
     }
 
-    // System.out.println((System.currentTimeMillis() - t0) + " msec for infix suggest");
+    // System.out.println((System.currentTimeMillis() - t0) + " ms for infix suggest");
     // System.out.println(results);
 
     return results;
