@@ -452,7 +452,7 @@ public class AnalyzingSuggester extends Lookup {
             payload = null;
           }
 
-          buffer = ArrayUtil.grow(buffer, requiredLength);
+          buffer = ArrayUtil.growNoCopy(buffer, requiredLength);
 
           output.reset(buffer);
 
@@ -500,7 +500,7 @@ public class AnalyzingSuggester extends Lookup {
       PairOutputs<Long, BytesRef> outputs =
           new PairOutputs<>(PositiveIntOutputs.getSingleton(), ByteSequenceOutputs.getSingleton());
       FSTCompiler<Pair<Long, BytesRef>> fstCompiler =
-          new FSTCompiler<>(FST.INPUT_TYPE.BYTE1, outputs);
+          new FSTCompiler.Builder<>(FST.INPUT_TYPE.BYTE1, outputs).build();
 
       // Build FST:
       BytesRefBuilder previousAnalyzed = null;
@@ -523,7 +523,7 @@ public class AnalyzingSuggester extends Lookup {
         }
         input.reset(bytes.bytes, bytes.offset, bytes.length);
         short analyzedLength = input.readShort();
-        analyzed.grow(analyzedLength + 2);
+        analyzed.growNoCopy(analyzedLength + 2);
         input.readBytes(analyzed.bytes(), 0, analyzedLength);
         analyzed.setLength(analyzedLength);
 
@@ -587,7 +587,7 @@ public class AnalyzingSuggester extends Lookup {
           fstCompiler.add(scratchInts.get(), outputs.newPair(cost, br));
         }
       }
-      fst = fstCompiler.compile();
+      fst = FST.fromFSTReader(fstCompiler.compile(), fstCompiler.getFSTReader());
       count = newCount;
 
       // Util.dotToFile(fst, "/tmp/suggest.dot");
@@ -613,12 +613,9 @@ public class AnalyzingSuggester extends Lookup {
   @Override
   public boolean load(DataInput input) throws IOException {
     count = input.readVLong();
-    this.fst =
-        new FST<>(
-            input,
-            input,
-            new PairOutputs<>(
-                PositiveIntOutputs.getSingleton(), ByteSequenceOutputs.getSingleton()));
+    PairOutputs<Long, BytesRef> outputs =
+        new PairOutputs<>(PositiveIntOutputs.getSingleton(), ByteSequenceOutputs.getSingleton());
+    this.fst = new FST<>(FST.readMetadata(input, outputs), input);
     maxAnalyzedPathsForOneInput = input.readVInt();
     hasPayloads = input.readByte() == 1;
     return true;
